/* ================================================================
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL DINGOO GAMES OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
 * IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ====================================================================
 */ 

#include "../inc/GameEngine.h"

#include "../game/spoutex.h"
#include "../game/piece.h"

int pad;
unsigned short temp[SPEX_WIDTH*SPEX_HEIGHT];

//--------------------------------------------------------------------------------------------------------
GameEngine::GameEngine( void* userdata [] ) : 
					Engine( userdata )
{
	m_pAudioData = NULL;
}
//--------------------------------------------------------------------------------------------------------
GameEngine::~GameEngine( void )
{
	Free();
}
//--------------------------------------------------------------------------------------------------------
/// init all resources
result	GameEngine::Init( void )
{
	InitDLRes();
	// 16 FPS
	SetFPS( 30 );

	//Init texture
	m_pImgS2d = new Texture();

	//Paint Area
	rcTest.left = 0;
	rcTest.top = 0;
	rcTest.right = 320;
	rcTest.bottom = 240;

	// Init game
	pceAppInit();

	return S2D_OK;
}
//--------------------------------------------------------------------------------------------------------
/// free all resources
void	GameEngine::Free( void )
{
	ReleaseDLRes();
}
//--------------------------------------------------------------------------------------------------------
/// game logic before render
void	GameEngine::Exec( void )
{  
	pceAppProc();

    JoyPadControl();
}
//--------------------------------------------------------------------------------------------------------
/// game logic after render
void	GameEngine::Exec2( void )
{
}
//--------------------------------------------------------------------------------------------------------
/// pre-render proccess
void	GameEngine::PreRender( void )
{

}
//--------------------------------------------------------------------------------------------------------
/// render
void	GameEngine::Render( void )
{
  //Pixel by pixel
  int x, y;
  Pixel *pECRAN = m_pDraw2D->GetVRAMPtr();
  for(y = 0; y < SPEX_HEIGHT; y ++) {
    for(x = 0; x < SPEX_WIDTH; x ++) {
	  s32 ofs = (y*SPEX_WIDTH) + (x);
      temp[ofs] = pltTbl[vBuffer[x+(y<<7)]];
    }
  }

  m_pImgS2d->CreateFromMemory(TF_TRUECOLOR,SPEX_WIDTH,SPEX_HEIGHT,&temp,sizeof(unsigned short),NULL);
  m_pDraw2D->SetTexture( m_pImgS2d );		
  m_pDraw2D->DrawStretchOverlay(rcTest,NULL);
  m_pImgS2d->Free();
}
//--------------------------------------------------------------------------------------------------------
void	GameEngine::PostRender( void )
{
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::DispDebugInfo( void )
{
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::EnterGame( s32 MapId )
{ 
}
//--------------------------------------------------------------------------------------------------------
void	GameEngine::PlayMusic(u32 ID,bool bRep)
{
	if(!m_bUseAudio){
		return;
	}
	if(ID>=m_AudioCnt){
		return;
	}
	m_pAudioPlayer->Play(&m_pAudioData[ID],0,m_pAudioData[ID].GetLength(),255,bRep);
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::StopMusic()
{
	m_pAudioPlayer->Stop(0);
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::PlaySound(u32 ID)
{
	if(!m_bUseAudio){
		return;
	}
	if(ID>=m_AudioCnt){
		return;
	}
	for(s32 i=1;i<AudioPlayer::MaxChannels;i++){
		if(m_pAudioPlayer->GetState(i) != AUDIO_PLAYING){
			m_pAudioPlayer->Play(&m_pAudioData[ID],i,m_pAudioData[ID].GetLength(),255,false);
			return;
		}
	}
}	

//--------------------------------------------------------------------------------------------------------
void	GameEngine::StopAllSound()
{
	m_bUseAudio = false;

	m_pAudioPlayer->StopAll();
}

//--------------------------------------------------------------------------------------------------------
void	GameEngine::ExitGame( void )
{
	Exit();
}

//--------------------------------------------------------------------------------------------------------
void GameEngine::LoadCfg(void)
{
	// get full filename with path
	Char fullname[256];
	s2d::strcpy( fullname, _LS(".\\lltconfig.sdt") );

	FILE* m_fp = fopen( fullname, _LS("rb") );

	if(m_fp == NULL){
		return;
	}

	// read file to memory
	fseek( m_fp, 0, SEEK_END );
	s32 size =  ftell(m_fp);

	s32* mem = (s32*)  malloc( size );
	s32* pstar = mem;

	fseek(m_fp, 0, SEEK_SET );
	fread( mem, size,m_fp );
	fclose(m_fp);

	s32 Scene = *(s32*)mem;

	free( pstar );
}
//--------------------------------------------------------------------------------------------------------
void GameEngine::SaveCfg(void)
{
	// get full filename with path
	Char fullname[256];
	s2d::strcpy( fullname, _LS(".\\lltconfig.sdt") );

	FILE* m_fp = fopen( fullname, _LS("wb+") );

	if(m_fp == NULL){
		return;
	}

	s32 Scene=5;
	fwrite(&Scene, sizeof(s32),m_fp );

	// close file
	fclose(m_fp);
}
//--------------------------------------------------------------------------------------------------------
void GameEngine::JoyPadControl(void)
{
  pad &= pad & 0xffff;
  int i = 0, op = pad & 0x00ff;

  Keys k[] = {
    KEY_UP,			KEY_DN,			KEY_LF,			KEY_RT,
    KEY_UP,			KEY_DN,			KEY_LF,			KEY_RT,
    KEY_1,			KEY_2,			KEY_1,			KEY_2,
    KEY_L2,			KEY_R2,			KEY_R2			
  };

  int p[] = {
    PAD_UP,			PAD_DN,			PAD_LF,			PAD_RI,
    PAD_UP,			PAD_DN,			PAD_LF,			PAD_RI,
    PAD_A,			PAD_B,			PAD_A,			PAD_B,
    PAD_C,			PAD_D,			PAD_D,
    -1
  };

  pad = 0;

  do {
	 if( m_pInput->IsKeyDown( k[i]) ){
      pad |= p[i];
    }
    i ++;
  } while(p[i] >= 0);
  pad |= (pad & (~op)) << 8;
  
  if ( m_pInput->IsKeyClick( KEY_R1 ) ){
	  if (gamePhase ==1) {
		gamePhase = 2;
	  } 
	  pad |= TRG_ST;
  }

  if ( m_pInput->IsKeyClick( KEY_4 ) ){
	 ExitGame();
  }
}